// ---------------------------------------------------------------------------
// zoom.cs -- Version 4.2 -- October 25, 2000
// by Lorne Laliberte (writer@planetstarsiege.com)
//
// - define any number of zoom levels (13 are set up by default)
// - enable or disable the zoom rollover at minimum and maximum zooms
// - toggle zoom mode on with a key tap
// - press a key to set a zoom level directly
// - add events for zoom on/off and zooming in/out (for scripters)
//
// http://www.writerscripts.com
// ---------------------------------------------------------------------------

include("writer\\version.cs");
version("writer\\zoom.cs", "4.2", "Lorne Laliberte", "Oct 25, 2000");

include("writer\\bind.cs");
include("writer\\event.cs");
include("writer\\save.cs");
include("writer\\schedule.cs");
include("writer\\tap.cs");
include("writer\\whatif.cs");

// ---------------------------------------------------------------------------
// DEFAULT SETTINGS:
//
// Edit these if you're running this script for the first time
// All subsequent settings must be done in the config\wp_zoom.cs file

    // IMPORTANT -- set this to match your normal field of view
    $zoom::normalFOV = "90";

    // Enable toggling the zoom by tapping the zoom key quickly
    $zoom::KeyTapEnabled = "false";

    // Taptime in seconds -- pressing and releasing the zoom key within this
    // amount of time will be considered a key "tap," enabling zoom mode until
    // the next press of the zoom key.  I find 0.075 seconds works well.
    $zoom::KeyTapTime = "0.075";

    // Set this to 1 to enable the zoom rollover by default, or set it to 0 to
    // disable the zoom rollover by default.
    $zoom::Rollover = "0";

    // Magnification for each zoom level -- you can set up as many zoom levels
    // as you want, but Tribes can only render magnifications between 1 and 20.
    // NOTE: place all values within quotes!
    $zoom::mag[0] = "1";
    $zoom::mag[1] = "1.5";
    $zoom::mag[2] = "2";
    $zoom::mag[3] = "2.5";
    $zoom::mag[4] = "3";
    $zoom::mag[5] = "3.5";
    $zoom::mag[6] = "4";
    $zoom::mag[7] = "5";
    $zoom::mag[8] = "6";
    $zoom::mag[9] = "8";
    $zoom::mag[10] = "10";
    $zoom::mag[11] = "12";
    $zoom::mag[12] = "20";

    // Total number of zoom levels listed above
    $zoom::levels = "13";

    // Zoom level ([0], [1] etc.) you want to use by default
    $zoom::default = "5";

    // Minimum and maximum zoom levels you want to use without overriding
    $zoom::min = "0";
    $zoom::max = "12";

    // Override your min/max zoom levels while this key is held down...
    // I like to use a modifier like "lshift" or "control"
    $zoom::OverrideKey = "";

    // Note: you can also set up specific keys to do the override by passing "true"
    // or "1" to the zoom::in::pressed() and zoom::out::pressed() functions (see below)

    // Delay in seconds before your zoom::in and zoom::out keys will start
    // repeating when held down
    $zoom::keyFirstDelay = "0.2";

    // Delay in seconds between each repeat of the zoom key (repeat rate)
    $zoom::keyRepeatDelay = "0.1";

    // Toggle zoom mode on/off
    //bind(keyboard0, make, "e", to, "zoom::Pressed();");
    //bind(keyboard0, break, "e", to, "zoom::Released();");

    // Set zoom levels directly (use id strings from zoom_set_1 to zoom_set_20)
    //bind(keyboard0, make, l_shift, "1", TO, "zoom::set(2);", zoom_set_1);
    //bind(keyboard0, make, l_shift, "2", TO, "zoom::set(4);", zoom_set_2);
    //bind(keyboard0, make, l_shift, "3", TO, "zoom::set(7);", zoom_set_3);
    //bind(keyboard0, make, l_shift, "4", TO, "zoom::set(9);", zoom_set_4);
    //bind(keyboard0, make, l_shift, "5", TO, "zoom::set(12);", zoom_set_5);
    //bind(keyboard0, make, l_alt, "z", TO, "zoom::set(4);", zoom_set_6);
    //bind(keyboard0, make, l_alt, "x", TO, "zoom::set(11);", zoom_set_7);

    // Toggle zoom rollover on and off
    //bind(keyboard0, make, control, "z", TO, "zoom::toggleRollover();");

    // Increase zoom
    //bind(keyboard0, make, "z", to, "zoom::in::pressed();");
    //bind(keyboard0, break, "z", to, "zoom::in::released();");

    // Decrease zoom
    //bind(keyboard0, make, "x", to, "zoom::out::pressed();");
    //bind(keyboard0, break, "x", to, "zoom::out::released();");

// ---------------------------------------------------------------------------

WriterPrefs::load("zoom");

function zoom::setup()
{
    for(%i = 0; %i < $zoom::levels; %i++)
        $zoom::fov[%i] = $zoom::normalFOV/$zoom::mag[%i];
}
zoom::setup();


function Zoom::Pressed()
{
    Event::Trigger(eventZoomPressed, $zoom::mag[$zoom::level]);

    if( !Event::Returned(eventZoomPressed, mute) )
    {
        $zoomed = true;
        $pref::PlayerFov = $zoom::fov[$zoom::level];
        Event::Trigger(eventZoomOn, $zoom::mag[$zoom::level]);

        tap(zoom, $zoom::KeyTapTime);
    }
}

function Zoom::Released()
{
    Event::Trigger(eventZoomReleased);

    if( !Event::Returned(eventZoomReleased, mute) )
    {
        $zoomed = false;
        $pref::PlayerFov = $zoom::normalFOV;
        Event::Trigger(eventZoomOff, $zoom::mag[$zoom::level]);
    }
}

function onZoomPressed()
{
    if($zoomed)
    {
        $zoomed = false;
        $pref::PlayerFov = $zoom::normalFOV;
        Event::Trigger(eventZoomOff, $zoom::mag[$zoom::level]);
        return mute;
    }
    return;
}

function onZoomReleased()
{
    if(wasTapped(zoom))
        return mute;

    return;
}


function zoom::set(%level)
{
    if((%level < 0) || (%level >= $zoom::levels))
        return;

    $zoom::level = %level;

    if($zoomed)
        $pref::PlayerFov = $zoom::fov[$zoom::level];

    Event::Trigger(eventZoomChanged, $zoom::mag[$zoom::level]);
}


// Toggle zoom rollover
//
function zoom::toggleRollover()
{
    if($zoom::rollover)
        $zoom::rollover = 0;
    else
        $zoom::rollover = 1;

    remoteBP(2048, "<L5>Zoom Rollover " @ wif($zoom::rollover, "ON", "OFF"), 4);
}


// Increase zoom level with option to stop at max zoom level
//
function zoom::in::pressed(%force)
{
    if($zoom::out::repeating)
    {
        $zoom::in::repeating = true;
        $zoom::out::repeating = false;
        schedule::cancel($zoom::out::repeating);
    }

    %max = $zoom::max;
    %min = $zoom::min;

    if(%force || $Modifier::Pressed[$zoom::OverrideKey])
    {
        %max = $zoom::levels - 1;
        %min = 0;
    }

    if($zoom::rollover && (($zoom::level + 1) > %max))
        %next = %min;
    else
        %next = $zoom::level + 1;

    if(%next > %max)
        return;

    Event::Trigger(eventZoomIn, $zoom::mag[%next], $zoom::mag[$zoom::level]);

    if(Event::Returned(eventZoomIn, mute))
        return;

    zoom::set(%next);

    if($zoom::in::repeating)
        schedule::add("Zoom::in::Pressed();", $zoom::keyRepeatDelay, zoom::in::repeating);
    else
        schedule::add("Zoom::in::Pressed();", $zoom::keyFirstDelay, zoom::in::repeating);

    $zoom::in::repeating = true;

}


function zoom::in::released()
{
    $zoom::in::repeating = false;
    schedule::cancel(zoom::in::repeating);
}

// Decrease zoom level with option to stop at min zoom
//
function zoom::out::pressed(%force)
{
    if($zoom::in::repeating)
    {
        $zoom::out::repeating = true;
        $zoom::in::repeating = false;
        schedule::cancel($zoom::in::repeating);
    }

    %max = $zoom::max;
    %min = $zoom::min;

    if(%force || $Modifier::Pressed[$zoom::OverrideKey])
    {
        %max = $zoom::levels - 1;
        %min = 0;
    }

    if($zoom::rollover && (($zoom::level - 1) < %min))
        %next = %max;
    else
        %next = $zoom::level - 1;

    if(%next < %min)
        return;

    Event::Trigger(eventZoomOut, $zoom::mag[%next], $zoom::mag[$zoom::level]);

    if(Event::Returned(eventZoomOut, mute))
        return;

    zoom::set(%next);

    if($zoom::out::repeating)
        schedule::add("Zoom::out::Pressed();", $zoom::keyRepeatDelay, zoom::out::repeating);
    else
        schedule::add("Zoom::out::Pressed();", $zoom::keyFirstDelay, zoom::out::repeating);

    $zoom::out::repeating = true;
}


function zoom::out::released()
{
    $zoom::out::repeating = false;
    schedule::cancel(zoom::out::repeating);
}


function zoom::save()
{
    AddExportText("    // IMPORTANT -- set this to match your normal field of view");
    ExportSetting("    $zoom::normalFOV", 90);
    AddExportText("");
    AddExportText("    // Enable toggling the zoom by tapping the zoom key quickly");
    ExportSetting("    $zoom::KeyTapEnabled", false);
    AddExportText("");
    AddExportText("    // Taptime in seconds -- pressing and releasing the zoom key within this");
    AddExportText("    // amount of time will be considered a key \"tap,\" enabling zoom mode until");
    AddExportText("    // the next press of the zoom key.  I find 0.075 seconds works well.");
    ExportSetting("    $zoom::KeyTapTime", "0.075");
    AddExportText("");
    AddExportText("    // Set this to 1 to enable the zoom rollover by default, or set it to 0 to");
    AddExportText("    // disable the zoom rollover by default.");
    ExportSetting("    $zoom::Rollover", 1);
    AddExportText("");
    AddExportText("    // Magnification for each zoom level -- you can set up as many zoom levels");
    AddExportText("    // as you want, but Tribes can only render magnifications between 1 and 20.");
    AddExportText("    // NOTE: place all values within quotes!");


    if($zoom::levels)
    {
        for(%i = 0; %i < $zoom::levels; %i++)
            AddExportText("    $zoom::mag[" @ %i @ "] = \"" @ $zoom::mag[%i] @ "\";");

        AddExportText("");
        AddExportText("    // Total number of zoom levels listed above");
        AddExportText("    $zoom::levels = \"" @ $zoom::levels @ "\";");
    }
    else
    {
        AddExportText("    $zoom::mag[0] = \"1\";");
        AddExportText("    $zoom::mag[1] = \"1.5\";");
        AddExportText("    $zoom::mag[2] = \"2\";");
        AddExportText("    $zoom::mag[3] = \"2.5\";");
        AddExportText("    $zoom::mag[4] = \"3\";");
        AddExportText("    $zoom::mag[5] = \"3.5\";");
        AddExportText("    $zoom::mag[6] = \"4\";");
        AddExportText("    $zoom::mag[7] = \"5\";");
        AddExportText("    $zoom::mag[8] = \"6\";");
        AddExportText("    $zoom::mag[9] = \"8\";");
        AddExportText("    $zoom::mag[10] = \"10\";");
        AddExportText("    $zoom::mag[11] = \"12\";");
        AddExportText("    $zoom::mag[12] = \"20\";");
        AddExportText("");
        AddExportText("    // Total number of zoom levels listed above");
        AddExportText("    $zoom::levels = 13;");
    }

    AddExportText("");
    AddExportText("    // Zoom level ([0], [1] etc.) you want to use by default");
    ExportSetting("    $zoom::default", 4);
    AddExportText("");
    AddExportText("    // Minimum and maximum zoom levels you want to use without overriding");
    ExportSetting("    $zoom::min", 0);
    ExportSetting("    $zoom::max", 12);
    AddExportText("");
    AddExportText("    // Override your min/max zoom levels while this key is held down");
    AddExportText("    // I like to use a modifier like \"lshift\" or \"control\"");
    ExportSetting("    $zoom::OverrideKey", "");
    AddExportText("");
    AddExportText("    // Note: you can also set up keys to do the override by passing \"true\"");
    AddExportText("    // or \"1\" to the zoom::in::pressed() and zoom::out::pressed() functions (see below)");
    AddExportText("");
    AddExportText("    // Delay in seconds before your zoom::in and zoom::out keys will start");
    AddExportText("    // repeating when held");
    ExportSetting("    $zoom::keyFirstDelay", "0.2");
    AddExportText("");
    AddExportText("    // Delay in seconds between each repeat of the zoom key (repeat rate)");
    ExportSetting("    $zoom::keyRepeatDelay", "0.1");
    AddExportText("");
    AddExportText("    // Toggle zoom mode on/off");

    ExportBind("zoom::Pressed();", "//bind(keyboard0, make, \"e\", TO, \"zoom::Pressed();\");");
    ExportBind("zoom::Released();", "//bind(keyboard0, break, \"e\", TO, \"zoom::Released();\");");

    AddExportText("");
    AddExportText("    // Set zoom levels directly (use id strings from zoom_set_1 to zoom_set_20)");

    ExportBind("zoom_set_1", "//bind(keyboard0, make, l_shift, \"1\", TO, \"zoom::set(2);\", zoom_set_1);");
    ExportBind("zoom_set_2", "//bind(keyboard0, make, l_shift, \"2\", TO, \"zoom::set(4);\", zoom_set_2);");
    ExportBind("zoom_set_3", "//bind(keyboard0, make, l_shift, \"3\", TO, \"zoom::set(7);\", zoom_set_3);");
    ExportBind("zoom_set_4", "//bind(keyboard0, make, l_shift, \"4\", TO, \"zoom::set(9);\", zoom_set_4);");
    ExportBind("zoom_set_5", "//bind(keyboard0, make, l_shift, \"5\", TO, \"zoom::set(12);\", zoom_set_5);");
    ExportBind("zoom_set_6", "//bind(keyboard0, make, l_alt, \"z\", TO, \"zoom::set(4);\", zoom_set_6);");
    ExportBind("zoom_set_7", "//bind(keyboard0, make, l_alt, \"x\", TO, \"zoom::set(11);\", zoom_set_7);");
    ExportBind("zoom_set_8", "", zoom_set_8);
    ExportBind("zoom_set_9", "", zoom_set_9);
    ExportBind("zoom_set_10", "", zoom_set_10);
    ExportBind("zoom_set_11", "", zoom_set_11);
    ExportBind("zoom_set_12", "", zoom_set_12);
    ExportBind("zoom_set_13", "", zoom_set_13);
    ExportBind("zoom_set_14", "", zoom_set_14);
    ExportBind("zoom_set_15", "", zoom_set_15);
    ExportBind("zoom_set_16", "", zoom_set_16);
    ExportBind("zoom_set_17", "", zoom_set_17);
    ExportBind("zoom_set_18", "", zoom_set_18);
    ExportBind("zoom_set_19", "", zoom_set_19);
    ExportBind("zoom_set_20", "", zoom_set_20);

    AddExportText("");
    AddExportText("    // Toggle zoom rollover on and off");

    ExportBind("zoom::toggleRollover();", "//bind(keyboard0, make, control, \"z\", TO, \"zoom::toggleRollover();\");");

    AddExportText("");
    AddExportText("    // Increase zoom");

    ExportBind("zoom::in::pressed();", "//bind(keyboard0, make, \"z\", TO, \"zoom::in::pressed();\");");
    ExportBind("zoom::in::released();", "//bind(keyboard0, break, \"z\", TO, \"zoom::in::released();\");");

    AddExportText("");
    AddExportText("    // Decrease zoom");

    ExportBind("zoom::out::pressed();", "//bind(keyboard0, make, \"x\", TO, \"zoom::out::pressed();\");");
    ExportBind("zoom::out::released();", "//bind(keyboard0, break, \"x\", TO, \"zoom::out::released();\");");
}

WriterPrefs::save("zoom");

function zoom::init()
{
    needs("writer\\zoom.cs", "writer\\bind.cs 1.3",
                             "writer\\event.cs 2.7",
                             "writer\\save.cs 2.3",
                             "writer\\schedule.cs 1.3",
                             "writer\\tap.cs 1.2",
                             "writer\\whatif.cs 1.1");

    // Reset zoom whenever we join a game
    Event::Attach(eventConnected, "zoom::set($zoom::default);");

    if($zoom::OverrideKey != "")
        Modifier::bind($zoom::OverrideKey);

    if($zoom::KeyTapEnabled)
    {
        Event::Attach(eventZoomPressed, "onZoomPressed();");
        Event::Attach(eventZoomReleased, "onZoomReleased();");
    }
}
Event::Attach(writerInit, "zoom::init();");

if(!$UsingWriterLoader) zoom::init();
